#include <Arduino.h>
#include <muses_72323.h>
#include <SPI.h>

#define MUSES_CS  5



muses_72323_command_t command;

muses_72323_chip_address_t            chip_address    = 0;
muses_72323_zero_window_t             zero_window     = MUSES_72323_MAX_ZERO_WINDOW;
muses_72323_soft_step_clock_divider_t clock_divider   = 1;
muses_72323_soft_step_clock_source_t  soft_step_clock = MUSES_72323_SOFT_STEP_CLOCK_INTERNAL;  

bool link_channels  = false;
bool use_soft_step  = false;
bool use_zero_cross = true;



// ------------------------------------------------------------------------------------
void setup() {
  Serial.begin(115200);
  pinMode(MUSES_CS, OUTPUT);
  digitalWrite(MUSES_CS, HIGH);  
  SPI.begin();
  SPI.beginTransaction(SPISettings(250000, MSBFIRST, SPI_MODE0));  // MUSES72323 SPI interface is rated at 250kHz maximum

  musesConfigure();
  musesMute(MUSES_72323_CHANNEL_RIGHT);
  musesMute(MUSES_72323_CHANNEL_LEFT);

}


// ------------------------------------------------------------------------------------
void loop()
{
  for (uint16_t i=0;i<=64;i++) {
    musesWriteVolume(MUSES_72323_CHANNEL_RIGHT, i);
    delay(500);
    musesWriteVolume(MUSES_72323_CHANNEL_LEFT, i);
    delay(500);
  }
  for (uint16_t i=64;i>=0;i--) {
    musesWriteVolume(MUSES_72323_CHANNEL_RIGHT, i);
    delay(500);
    musesWriteVolume(MUSES_72323_CHANNEL_LEFT, i);
    delay(500);
  }
  delay(1000); 
}



// ------------------------------------------------------------------------------------
// attenuation 0 to -480

void musesWriteVolume (muses_72323_channel_t channel, uint16_t attenuation) {
  muses_72323_error_t error = muses_72323_set_volume(&command, chip_address, channel, (muses_72323_attenuation_t)attenuation, use_soft_step);
  if (error) Serial.println(error);
  musesSend(command);
}



// ------------------------------------------------------------------------------------
// gain 0 to 7

void musesWriteGain (muses_72323_channel_gain_t left, muses_72323_channel_gain_t right) {
  muses_72323_error_t error = muses_72323_set_gain(&command, chip_address, left, right, link_channels, use_zero_cross);
  if (error) Serial.println(error);
  musesSend(command);
}


// ------------------------------------------------------------------------------------
void musesMute (muses_72323_channel_t channel) {
  muses_72323_error_t error = muses_72323_mute(&command, chip_address, channel, use_zero_cross);
  if (error) Serial.println(error);
  musesSend(command);
}


// ------------------------------------------------------------------------------------
void musesConfigure(void) {
  muses_72323_configure(&command, chip_address, zero_window, clock_divider, soft_step_clock);
  musesSend(command);
}


// ------------------------------------------------------------------------------------
void musesSend(muses_72323_command_t command) {
  Serial.print(highByte(command),BIN);
  Serial.print("\t");
  Serial.println(lowByte(command),BIN);

  digitalWrite(MUSES_CS, LOW);
  delayMicroseconds(1);   
  SPI.transfer(highByte(command));
  SPI.transfer(lowByte(command));  
  delayMicroseconds(1);   
  digitalWrite(MUSES_CS, HIGH);
}





// ---------------------- the end ----------------------------
